#!/usr/bin/python

# Template for 1st programming assignment
# DB2 - WS 2015/16

# IMPORTS:

# module to read linewise from standard input
import fileinput

# to access command-line parameter and standard input (see below)
import sys


# this is the main function
def main():
    # initialize a list
    # NOTE: this is similar to a Vector in Java
    tracklist = []

    # Loop over each line from standard input
    # NOTE: In python indention matters. We follow Python's style guide in 
    # this example, which uses 4 whitespace characters for an indention level.
    for line in fileinput.input("-"):

        # There is a newline at the end of each line - get rid of it
        line_stripped = line.strip()

        # convert explicitly to ints to get track number
        tracknumber = int(line_stripped)

        # add to tracklist
        tracklist.append(tracknumber)
        

    # assign first command-line argument to variable algo
    # NOTE: as in Java, the first element of an array (list in python)
    # is accessed via index 0. However, sys.argv[0] contains the name python
    # script, not the first cmdline parameter
    algo = sys.argv[1]

    # decide which algorithm to execute
    if algo == "fcfs":
        track_order = fcfs(tracklist)
    elif algo == "sstf":
        track_order = sstf(tracklist)
    elif algo == "look":
        track_order = look(tracklist)
    else:
        raise Exception("Unknown algorithm: %s" % algo)

    tracks_traversed = compute_cost(track_order)

    # Output
    for track in track_order:
        # Print track. print adds a newline automatically
        print track
    print "%d tracks traversed" % tracks_traversed



# Function implementing First Come First Served
def fcfs(tracklist):

    return tracklist


# Function implementing Shortest Seek Time First
def sstf(tracklist):

    # get starting track as cur_track
    cur_track = tracklist[0]

    # we put starting track to the output track list
    tracklist_sstf = [ cur_track ]

    # We solve the problem as follows:
    # We sort track list and remove already handled tracks

    # sort track list
    # this ensures that the only tracks to compare with are before and after the current track
    tracklist_sorted = sorted(tracklist)

    # find index of the starting track in the sorted list (due to sorting, it 
    # may no longer be the first element)
    for i in range(len(tracklist_sorted)):
        if cur_track == tracklist_sorted[i]:
            cur_track_id = i

    # delete the current track from the sorted list
    del tracklist_sorted[cur_track_id]

    # while there are elements in the sorted track list
    while len(tracklist_sorted) != 0:

        # compute difference to the neighboring tracks

        # first, the next track. We need to ignore it, if there is no next track.
        diffup = sys.maxint
        if cur_track_id != len(tracklist_sorted):
            diffup = abs(cur_track - tracklist_sorted[cur_track_id])

        # second, the previous track. We need to ignore it, if there is no previous track.
        diffdown = sys.maxint
        if cur_track_id != 0:
            diffdown = abs(cur_track - tracklist_sorted[cur_track_id - 1])

        # decide which track is closer to the current track
        # and make the closer track the current track
        if diffup <= diffdown:
            cur_track = tracklist_sorted[cur_track_id]
        else:
            cur_track = tracklist_sorted[cur_track_id - 1]
            cur_track_id = cur_track_id - 1

        # append the closer track (current track) to the output track list
        tracklist_sstf.append(cur_track)

        # delete the closer track from the sorted list
        del tracklist_sorted[cur_track_id]
        

    return tracklist_sstf
    

# Function implementing LOOK
def look(tracklist):
    
    raise Exception("Look is not implemented")

    # Assume that the first direction is up

    # do not forget to return a result list
    

# Compute number of tracks traversed by the disk head
def compute_cost(tracklist_algo):

    # initialize tracks_traversed
    tracks_traversed = 0

    # iterate over track list
    # i will get values from 1 to len(tracklist) - 1
    # so we collect the difference between i - 1 and i
    for i in range(1, len(tracklist_algo)):

        # compute absolute difference between two consecutive track numbers
        absdiff = abs(tracklist_algo[i] - tracklist_algo[i - 1])

        # add to tracks_traversed
        tracks_traversed += absdiff

    return tracks_traversed


# the main function needs to be called explicitly
if __name__ == "__main__":
    main()
